@@ -19,8 +19,10 @@ def _get_storage_client(http):
     # Cloud storage uses environment variables to configure api endpoints for
     # file upload - which is read at module import time
     from google.cloud import storage
+
     if os.getenv("DEBUG"):
         from http import client as http_client
+
         http_client.HTTPConnection.debuglevel = 5
     return storage.Client(
         project="[PROJECT]",
@@ -30,7 +32,6 @@ def _get_storage_client(http):
 
 
 class ServerBaseCase(BaseTestCase):
-
     @classmethod
     def setUpClass(cls):
         cls._server = create_server("localhost", 9023, in_memory=False)
@@ -47,7 +48,6 @@ class ServerBaseCase(BaseTestCase):
 
 
 class BucketsTests(BaseTestCase):
-
     @classmethod
     def setUpClass(cls):
         cls._server = create_server("localhost", 9023, in_memory=False)
@@ -133,7 +133,8 @@ class BucketsTests(BaseTestCase):
         self.assertIsNone(blob)
 
         with fs.open_fs(STORAGE_BASE + STORAGE_DIR) as pwd:
-            self.assertFalse(pwd.exists('bucket_name'))
+            self.assertFalse(pwd.exists("bucket_name"))
+
     # TODO: test delete-force
 
 
@@ -144,7 +145,9 @@ class DefaultBucketTests(BaseTestCase):
         return super().tearDown()
 
     def test_bucket_created(self):
-        self._server = create_server("localhost", 9023, in_memory=True, default_bucket="example.appspot.com")
+        self._server = create_server(
+            "localhost", 9023, in_memory=True, default_bucket="example.appspot.com"
+        )
         self._server.start()
         self._session = requests.Session()
         self._client = _get_storage_client(self._session)
@@ -152,7 +155,6 @@ class DefaultBucketTests(BaseTestCase):
 
 
 class ObjectsTests(ServerBaseCase):
-
     def test_upload_from_string(self):
         content = "this is the content of the file\n"
         bucket = self._client.create_bucket("testbucket")
@@ -164,7 +166,9 @@ class ObjectsTests(ServerBaseCase):
             self.assertEqual(read_content, content)
 
     def test_upload_from_text_file(self):
-        text_test = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_text.txt')
+        text_test = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), "test_text.txt"
+        )
         bucket = self._client.create_bucket("testbucket")
         blob = bucket.blob("test_text.txt")
         with open(text_test, "rb") as file:
@@ -178,7 +182,9 @@ class ObjectsTests(ServerBaseCase):
             self.assertEqual(read_content, expected_content)
 
     def test_upload_from_bin_file(self):
-        test_binary = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_binary.png')
+        test_binary = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), "test_binary.png"
+        )
         bucket = self._client.create_bucket("testbucket")
         blob = bucket.blob("binary.png")
         with open(test_binary, "rb") as file:
@@ -192,14 +198,12 @@ class ObjectsTests(ServerBaseCase):
             self.assertEqual(read_content, expected_content)
 
     def test_upload_from_bin_file_cr_lf(self):
-        content = b'\r\rheeeeei\r\n'
+        content = b"\r\rheeeeei\r\n"
         test_binary = BytesIO(content)
         bucket = self._client.create_bucket("testbucket")
         blob = bucket.blob("binary_cr.png")
 
-        blob.upload_from_file(
-            test_binary, size=len(content)
-        )
+        blob.upload_from_file(test_binary, size=len(content))
 
         with fs.open_fs(STORAGE_BASE + STORAGE_DIR) as pwd:
             read_content = pwd.readbytes("testbucket/binary_cr.png")
@@ -207,7 +211,9 @@ class ObjectsTests(ServerBaseCase):
         self.assertEqual(read_content, content)
 
     def test_upload_from_file_name(self):
-        test_binary = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_binary.png')
+        test_binary = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), "test_binary.png"
+        )
         file_name = "test_binary.png"
 
         bucket = self._client.create_bucket("testbucket")
@@ -239,7 +245,7 @@ class ObjectsTests(ServerBaseCase):
         blob = bucket.get_blob(file_name)
         self.assertEqual(blob.name, file_name)
 
-    def test_get_nonexistant(self):
+    def test_get_nonexistent(self):
         bucket = self._client.create_bucket("testbucket")
         res = bucket.get_blob("idonotexist")
 
@@ -260,7 +266,7 @@ class ObjectsTests(ServerBaseCase):
 
         blob = bucket.get_blob("iexist")
         fetched_content = blob.download_as_bytes()
-        self.assertEqual(fetched_content, content.encode('utf-8'))
+        self.assertEqual(fetched_content, content.encode("utf-8"))
 
     def test_set_content_encoding(self):
         content = "The quick brown fox jumps over the lazy dog\n"
@@ -275,7 +281,7 @@ class ObjectsTests(ServerBaseCase):
     def test_set_metadata(self):
         content = "The quick brown fox jumps over the lazy dog\n"
         bucket = self._client.create_bucket("testbucket")
-        metadata = {"Color": 'Pink'}
+        metadata = {"Color": "Pink"}
 
         blob = bucket.blob("testblob")
         blob.metadata = metadata
@@ -382,7 +388,9 @@ class ObjectsTests(ServerBaseCase):
             blob.upload_from_string(content)
 
     def test_download_binary_to_file(self):
-        test_binary = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_binary.png')
+        test_binary = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), "test_binary.png"
+        )
         bucket = self._client.create_bucket("testbucket")
 
         blob = bucket.blob("binary.png")
@@ -397,7 +405,9 @@ class ObjectsTests(ServerBaseCase):
             self.assertEqual(fetched_file.getvalue(), file.read())
 
     def test_download_text_to_file(self):
-        test_text = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_text.txt')
+        test_text = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), "test_text.txt"
+        )
         bucket = self._client.create_bucket("testbucket")
 
         blob = bucket.blob("text.txt")
@@ -423,6 +433,13 @@ class ObjectsTests(ServerBaseCase):
             self.assertIsNone(bucket.get_blob("cantouchme.txt"))
             self.assertFalse(pwd.exists("bucket_name/canttouchme.txt"))
 
+    def test_delete_nonexistent_object(self):
+        bucket = self._client.create_bucket("bucket_name")
+        blob = bucket.blob("this-should-not-exists.txt")
+
+        with self.assertRaises(NotFound):
+            blob.delete()
+
     def test_create_within_directory(self):
         bucket = self._client.create_bucket("bucket_name")
         blob = bucket.blob("this/is/a/nested/file.txt")
@@ -444,9 +461,7 @@ class ObjectsTests(ServerBaseCase):
             self.assertTrue(pwd.exists("bucket_name/this/is/a/nested/file.txt"))
 
     def _assert_blob_list(self, expected, actual):
-        self.assertEqual(
-            [b.name for b in expected],
-            [b.name for b in actual])
+        self.assertEqual([b.name for b in expected], [b.name for b in actual])
 
     def test_list_blobs_on_nonexistent_bucket(self):
         blobs = self._client.list_blobs("bucket_name")
@@ -487,7 +502,7 @@ class ObjectsTests(ServerBaseCase):
         blob_3 = bucket.blob("b/c.txt")
         blob_3.upload_from_string("text")
 
-        blobs = self._client.list_blobs(bucket, prefix='a')
+        blobs = self._client.list_blobs(bucket, prefix="a")
 
         self._assert_blob_list(blobs, [blob_1, blob_2])
 
@@ -506,9 +521,10 @@ class ObjectsTests(ServerBaseCase):
         blob_4 = bucket.blob("b/c.txt")
         blob_4.upload_from_string("text")
 
-        blobs = self._client.list_blobs(bucket, prefix='a', delimiter='/')
+        blobs = self._client.list_blobs(bucket, prefix="a/", delimiter="/")
 
         self._assert_blob_list(blobs, [blob_1, blob_2])
+        self.assertEqual(blobs.prefixes, {"a/b/"})
 
     def test_bucket_copy_existing(self):
         bucket = self._client.create_bucket("bucket_name")
@@ -529,6 +545,50 @@ class ObjectsTests(ServerBaseCase):
         with self.assertRaises(NotFound):
             bucket.rename_blob(blob_1, "c/d.txt")
 
+    def test_compose_create_new_blob(self):
+        bucket = self._client.create_bucket("compose_test")
+        data_1 = b"AAA\n"
+        source_1 = bucket.blob("source-1")
+        source_1.upload_from_string(data_1, content_type="text/plain")
+
+        data_2 = b"BBB\n"
+        source_2 = bucket.blob("source-2")
+        source_2.upload_from_string(data_2, content_type="text/plain")
+
+        destination = bucket.blob("destination")
+        destination.content_type = "text/somethingelse"
+        destination.compose([source_1, source_2])
+
+        composed = destination.download_as_bytes()
+        self.assertEqual(composed, data_1 + data_2)
+        self.assertEqual(destination.content_type, "text/somethingelse")
+
+    def test_compose_wo_content_type_set(self):
+        bucket = self._client.create_bucket("compose_test")
+        data_1 = b"AAA\n"
+        source_1 = bucket.blob("source-1")
+        source_1.upload_from_string(data_1, content_type="text/plain")
+
+        data_2 = b"BBB\n"
+        source_2 = bucket.blob("source-2")
+        source_2.upload_from_string(data_2, content_type="text/plain")
+
+        destination = bucket.blob("destination")
+        destination.compose([source_1, source_2])
+
+        composed = destination.download_as_bytes()
+        self.assertEqual(composed, data_1 + data_2)
+        self.assertEqual(destination.content_type, "text/plain")
+
+    def test_compose_nonexistent(self):
+        bucket = self._client.create_bucket("compose_test")
+        source_1 = bucket.blob("source-1")
+        source_2 = bucket.blob("source-2")
+
+        destination = bucket.blob("destination")
+        with self.assertRaises(NotFound):
+            destination.compose([source_1, source_2])
+
 
 class HttpEndpointsTest(ServerBaseCase):
     """ Tests for the HTTP endpoints defined by server.HANDLERS. """
@@ -546,11 +606,10 @@ class HttpEndpointsTest(ServerBaseCase):
         url = self._url("/anotherbucket/something.txt")
         response = requests.get(url)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, content.encode('utf-8'))
+        self.assertEqual(response.content, content.encode("utf-8"))
 
     def test_download_file_within_folder(self):
-        """ Cloud Storage allows folders within buckets, so the download URL should allow for this.
-        """
+        """Cloud Storage allows folders within buckets, so the download URL should allow for this."""
         content = "Here is some content"
         bucket = self._client.create_bucket("yetanotherbucket")
         blob = bucket.blob("folder/contain~ing/something~v-1.0.α.txt")
@@ -559,4 +618,4 @@ class HttpEndpointsTest(ServerBaseCase):
         url = self._url("/yetanotherbucket/folder/contain~ing/something~v-1.0.α.txt")
         response = requests.get(url)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content, content.encode('utf-8'))
+        self.assertEqual(response.content, content.encode("utf-8"))
